;; Generated package description from xref-union.el  -*- no-byte-compile: t -*-
(define-package "xref-union" "0.1.1.0.20230325.142012" "Combine multiple Xref backends" '((emacs "25.1")) :commit "0f98bea5948cee35aa9070b631bc0188159d5b3d" :authors '(("Philip Kaludercic" . "philipk@posteo.net")) :maintainer '("Philip Kaludercic" . "~pkal/public-inbox@lists.sr.ht") :url "https://git.sr.ht/~pkal/xref-union/")
